﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace SelfHostWebGet
{
    class Program
    {
        static void Main(string[] args)
        {
            var baseAddress1 = new Uri("http://" + Environment.MachineName + ":5555");
            var baseAddress2 = new Uri("https://" + Environment.MachineName + ":6666");
            var baseAddress3 = new Uri("net.tcp://" + Environment.MachineName + ":7777");
            Uri[] baseAddresses = { baseAddress1, baseAddress2, baseAddress3 };

            using (var host = new ServiceHost(typeof(SelfHostedSoapBasedServices), baseAddresses))
            {
                try
                {
                    // Enable publishing service for Silverlight
                    host.AddServiceEndpoint(typeof (ServiceContracts.IEcho), new BasicHttpBinding(), "basic");

                    // Enable publishing clientaccesspolicy.xml
                    host.AddServiceEndpoint(typeof(ServiceContracts.IPolicy), new WebHttpBinding(), "").Behaviors.Add(new WebHttpBehavior());
                    host.AddServiceEndpoint(typeof(ServiceContracts.IPolicy), new WSHttpBinding(), "secure");
                    host.AddServiceEndpoint(typeof(ServiceContracts.IPolicy), new NetTcpBinding(), "");

                    //// Enable publishing test.xml
                    //host.AddServiceEndpoint(typeof(ServiceContracts.IGetTestXml), bindingObject, "").Behaviors.Add(new WebHttpBehavior());

                    var bindingObject = new WebHttpBinding();

                    //// Enable publishing RESTful GetCustomer
                    host.AddServiceEndpoint(typeof (ServiceContracts.IRestFulCustomer), bindingObject, "rest").Behaviors.Add(new WebHttpBehavior());

                    // Enable metadata publishing.
                    var smb = new ServiceMetadataBehavior();
                    smb.HttpGetEnabled = true;
                    smb.MetadataExporter.PolicyVersion = PolicyVersion.Policy15;
                    host.Description.Behaviors.Add(smb);

                    // Open the host
                    host.Open();

                    //Console.WriteLine("The service is ready at {0}", baseAddress1);
                    Console.WriteLine("The service is ready at:");
                    foreach (var s in host.BaseAddresses)
                    {
                        Console.WriteLine(s.AbsolutePath + "   " + s.AbsoluteUri);
                    }
                    Console.WriteLine("Press <Enter> to terminate the service.");
                    Console.ReadLine();

                    // Close the ServiceHost.
                    host.Close();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
        }
    }
}
