﻿using System;
using System.Collections.Generic;
using System.IO;
using System.ServiceModel;
using System.ServiceModel.Web;

namespace SelfHostWebGet
{
    public class ServiceContracts
    {
        [ServiceContract]
        public interface IPolicy
        {
            [OperationContract]
            [WebGet(UriTemplate = "clientaccesspolicy.xml")]
            Stream GetSilverlightPolicy();

            [OperationContract]
            [WebGet(UriTemplate = "crossdomain.xml")]
            Stream GetCrossDomain();
        }

        [ServiceContract]
        public interface IEcho
        {
            [OperationContract]
            string Echo(string text);
        }
        
        [ServiceContract]
        public interface IRestFulCustomer
        {
            [OperationContract]
            [WebGet(UriTemplate = "/customer/{id}")]
            Customer GetCustomer(string id);
            
            [OperationContract]
            [WebInvoke(Method = "PUT", UriTemplate = "/customer", RequestFormat = WebMessageFormat.Xml, BodyStyle = WebMessageBodyStyle.Bare)]
            void PutCustomer(Customer customer);

            [OperationContract]
            [WebInvoke(Method = "DELETE", UriTemplate = "/customer/{id}")]
            void DeleteCustomer(string id);

            // #1
            [OperationContract]
            [WebGet(UriTemplate = "Product/{productIdString}", ResponseFormat = WebMessageFormat.Xml, BodyStyle = WebMessageBodyStyle.Bare)]
            Product FindProduct1(string productIdString);

            // #2
            [OperationContract]
            [WebGet(UriTemplate = "Product2/{productIdString=1011}", ResponseFormat = WebMessageFormat.Xml, BodyStyle = WebMessageBodyStyle.Bare)]
            Product FindProduct2(string productIdString);

            // #3
            [OperationContract]
            [WebGet(UriTemplate = "Product3/{productIdString}", ResponseFormat = WebMessageFormat.Xml, BodyStyle = WebMessageBodyStyle.Bare)]
            Product FindProduct3(string productIdString);

            // #4
            [OperationContract]
            [WebGet(UriTemplate = "Product/Name/{productName}", ResponseFormat = WebMessageFormat.Xml, BodyStyle = WebMessageBodyStyle.Bare)]
            Product FindProductByNameAsXml(string productName);

            // #5
            [OperationContract]
            [WebGet(UriTemplate = "Product/Name/{productName}?json", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Bare)]
            Product FindProductByNameAsJson(string productName);

            // #6
            [OperationContract]
            [WebGet(UriTemplate = "Product/xml?count={countString}&maxPrice={maxPriceString}", ResponseFormat = WebMessageFormat.Xml, BodyStyle = WebMessageBodyStyle.Bare)]
            List<Product> FindProductListAsXml(string countString, string maxPriceString);

            // #7
            [OperationContract]
            [WebGet(UriTemplate = "Product/json?count={countString}&maxPrice={maxPriceString}", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Bare)]
            List<Product> FindProductListAsJson(string countString, string maxPriceString);

            // # 999
            [OperationContract]
            [WebGet(UriTemplate = "Product8({productIdString})?json", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Bare)]
            Product FindProduct8(string productIdString);


            // # 9
            [OperationContract]
            [WebInvoke(UriTemplate = "Product?json", Method = "POST", RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Bare)]
            void AddProductAsJson(Product product);

            // # 10
            [OperationContract]
            [WebInvoke(UriTemplate = "Product", Method = "POST", RequestFormat = WebMessageFormat.Xml, BodyStyle = WebMessageBodyStyle.Bare)]
            void AddProductAsXml(Product product);

        }
    }
}
