﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Json;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Xml.Linq;
using SelfHostWebGet;

namespace SilverlightApplicationForSelfHost
{
    public partial class MainPage : UserControl
    {
        private readonly string _domain = "pumpkin:5555";
        private List<RESTfulOperation> _operations;

        public MainPage()
        {
            InitializeComponent();

            _operations = new List<RESTfulOperation>
                              {
                                  new RESTfulOperation{OperationNumber = 1, MethodName = "GetProductAsync_AsXml_IdInSegment", HTTPMethod = "GET", MessageDetail = "Response = XML", Description = "Id parameter is in its own segment of the Uri"},
                                  new RESTfulOperation{OperationNumber = 2, MethodName = "GetProductAsync_AsXml_DefaultValue", HTTPMethod = "GET", MessageDetail = "Response = XML", Description = "Id parameter has a default value"},
                                  new RESTfulOperation{OperationNumber = 3, MethodName = "GetProductAsync_AsXml_WithStatusCodes", HTTPMethod = "GET", MessageDetail = "Response = XML", Description = "Returns appropriate HTTP Status Codes"},
                                  new RESTfulOperation{OperationNumber = 4, MethodName = "GetProductByNameAsync_AsXml", HTTPMethod = "GET", MessageDetail = "Response = XML", Description = "Get's a product by name"},
                                  new RESTfulOperation{OperationNumber = 5, MethodName = "GetProductByNameAsync_AsJson", HTTPMethod = "GET", MessageDetail = "Response = JSON", Description = "Get's a product by name"},
                                  new RESTfulOperation{OperationNumber = 6, MethodName = "GetProductListAsync_AsXml", HTTPMethod = "GET", MessageDetail = "Response = XML", Description = "Get a list of products using parameters"},
                                  new RESTfulOperation{OperationNumber = 7, MethodName = "GetProductListAsync_AsJson", HTTPMethod = "GET", MessageDetail = "Response = JSON", Description = "Get a list of products using parameters"},
                                  new RESTfulOperation{OperationNumber = 8, MethodName = "GetProductListAsync_AsJson_DataContractJsonSerializer", HTTPMethod = "GET", MessageDetail = "Response = JSON", Description = "Get a list of products as JSON and deserialize using DataContractJsonSerializer"},
                                  new RESTfulOperation{OperationNumber = 9, MethodName = "PostProductAsync_AsJson_DataContractJsonSerializer", HTTPMethod = "POST", MessageDetail = "Request= JSON", Description = "Send a product as serialized JSON using DataContractJsonSerializer"},
                                  new RESTfulOperation{OperationNumber = 10, MethodName = "PostProductAsync_AsXml_DataContractSerializer", HTTPMethod = "POST", MessageDetail = "Request= XML", Description = "Send a product as serialized XML using DataContractSerializer"},

                                  new RESTfulOperation{OperationNumber = 999, MethodName = "GetProductAsync_AsJson_CompoundSegment", HTTPMethod = "GET", MessageDetail = "Response = JSON", Description = "Get a product using parameter in a compound segment"}
                              };
            Loaded += new RoutedEventHandler(Page_Loaded);
            btnRun.Click += ExecuteAsyncOperation;
        }

        public string BaseUri
        {
            get { return string.Format("http://{0}/rest", _domain); }
        }

        private void ExecuteAsyncOperation(object sender, RoutedEventArgs e)
        {
            var op = lstRESTfulActions.SelectedItem as RESTfulOperation;
            switch (op.OperationNumber)
            {
                case 1:
                    GetProductAsync_AsXml_IdInSegment();
                    break;
                case 2:
                    GetProductAsync_AsXml_DefaultValue();
                    break;
                case 3:
                    GetProductAsync_AsXml_WithStatusCodes();
                    break;
                case 4:
                    GetProductByNameAsync_AsXml();
                    break;
                case 5:
                    GetProductByNameAsync_AsJson();
                    break;
                case 6:
                    GetProductListAsync_AsXml();
                    break;
                case 7:
                    GetProductListAsync_AsJson();
                    break;
                case 8:
                    GetProductAsync_AsJson_DataContractJsonSerializer();
                    break;
                case 9:
                    PostProductAsync_AsJson_DataContractJsonSerializer();
                    break;
                case 10:
                    PostProductAsync_AsXml_DataContractSerializer();
                    break;

                case 999:
                    GetProductAsync_AsJson_CompoundSegment();
                    break;
            }
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            lstRESTfulActions.DataContext = _operations;
        }

        // # 1
        private void GetProductAsync_AsXml_IdInSegment()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsXml;
            var productId = 1001;
            var urlString = string.Format("{0}/Product/{1}", BaseUri, productId);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 2
        private void GetProductAsync_AsXml_DefaultValue()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsXml;
            var urlString = string.Format("{0}/Product2", BaseUri);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 3
        private void GetProductAsync_AsXml_WithStatusCodes()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsXml;
            var productId = 1001;
            var urlString = string.Format("{0}/Product3/{1}", BaseUri, productId);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 4
        private void GetProductByNameAsync_AsXml()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsXml;
            var productName = "Foo";
            var urlString = string.Format("{0}/Product/Name/{1}", BaseUri, productName);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 5
        private void GetProductByNameAsync_AsJson()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsJson;
            var productName = "Hoo";
            var urlString = string.Format("{0}/Product/Name/{1}?json", BaseUri, productName);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 6
        private void GetProductListAsync_AsXml()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsXml;
            var count = 10;
            decimal price = 50;
            var urlString = string.Format("{0}/Product/xml?count={1}&maxPrice={2}", BaseUri, count, price);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 7
        private void GetProductListAsync_AsJson()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsJson;
            var count = 10;
            decimal price = 50;
            var urlString = string.Format("{0}/Product/json?&count={1}&maxPrice={2}", BaseUri, count, price);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 8
        private void GetProductAsync_AsJson_DataContractJsonSerializer()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsJson_UsingDataContractJsonSerializer;
            var count = 10;
            decimal price = 50;
            var urlString = string.Format("{0}/Product/json?&count={1}&maxPrice={2}", BaseUri, count, price);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 999
        private void GetProductAsync_AsJson_CompoundSegment()
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += ParseProducts_AsJson;
            var productId = 1011;
            var urlString = string.Format("{0}/Product8({1})?json", BaseUri, productId);
            var uri = new Uri(urlString);
            wc.DownloadStringAsync(uri);
        }

        // # 9
        private void PostProductAsync_AsJson_DataContractJsonSerializer()
        {
            // Serialize a Product to JSON
            var product = new Product
                  {
                      ProductId = 301,
                      ProductName = "Another Thing",
                      UnitPrice = (decimal)19.99
                  };
            var serializer = new DataContractJsonSerializer(typeof(Product));
            var stream = new MemoryStream();
            serializer.WriteObject(stream, product);
            var jsonSerializedProduct = Encoding.UTF8.GetString(stream.ToArray(), 0, (int)stream.Length);


            // POST the JSON to the Async Operation
            var wc = new WebClient();
            wc.UploadStringCompleted += wc_UploadStringCompleted;
            wc.Headers["Content-type"] = "application/json";
            wc.Encoding = Encoding.UTF8;
            var urlString = string.Format("{0}/Product?json", BaseUri);
            wc.UploadStringAsync(new Uri(urlString), "POST", jsonSerializedProduct);
        }

        // # 10
        private void PostProductAsync_AsXml_DataContractSerializer()
        {
            // Serialize a Product to XML
            var product = new Product
            {
                ProductId = 301,
                ProductName = "Another Thing",
                UnitPrice = (decimal)19.99
            };
            var serializer = new DataContractSerializer(typeof(Product));
            var stream = new MemoryStream();
            serializer.WriteObject(stream, product);
            var xmlSerializedProduct = Encoding.UTF8.GetString(stream.ToArray(), 0, (int)stream.Length);


            // POST the XML to the Async Operation
            var wc = new WebClient();
            wc.UploadStringCompleted += wc_UploadStringCompleted;
            wc.Headers["Content-type"] = "application/xml";
            wc.Encoding = Encoding.UTF8;
            var urlString = string.Format("{0}/Product", BaseUri);
            wc.UploadStringAsync(new Uri(urlString), "POST", xmlSerializedProduct);
        }

        private void ParseProducts_AsJson_UsingDataContractJsonSerializer(
            object sender, DownloadStringCompletedEventArgs e)
        {
            string json = e.Result;
            var serializer = new DataContractJsonSerializer(typeof(List<Product>));
            //DataContractJsonSerializer serializer = new DataContractJsonSerializer(products.GetType());
            var stream = new MemoryStream(Encoding.UTF8.GetBytes(json));
            var products = serializer.ReadObject(stream) as List<Product>;
            stream.Close();
            lstProducts.DataContext = products;
        }

        // Gets product info from RESTful service
        private void ParseProducts_AsXml(object sender, DownloadStringCompletedEventArgs e)
        {
            var ns = "";
            var rawXml = e.Result;
            var xdoc = XDocument.Parse(rawXml);
            int result;
            decimal dec;
            var query = from product in xdoc.Descendants(ns + "Product")
                        select new Product
                                   {
                                       ProductId = int.TryParse(product.Element("ProductId").Value, out result) ? result : 0,
                                       ProductName = product.Element("ProductName").Value,
                                       UnitPrice = decimal.TryParse(product.Element("UnitPrice").Value, out dec) ? dec : 0
                                   };
            var products = query.ToList() as List<Product>;
            lstProducts.DataContext = products;
        }

        private void ParseProducts_AsJson(object sender, DownloadStringCompletedEventArgs e)
        {
            var raw = e.Result;

            JsonArray json;
            if (JsonArray.Parse(raw) as JsonArray == null)
                json = new JsonArray { JsonObject.Parse(raw) as JsonObject };
            else
                json = JsonArray.Parse(raw) as JsonArray;

            var query = from product in json
                        select new Product
                                   {
                                       ProductId = (int)product["ProductId"],
                                       ProductName = (string)product["ProductName"],
                                       UnitPrice = (decimal)product["UnitPrice"]
                                   };
            var products = query.ToList() as List<Product>;
            lstProducts.DataContext = products;
        }

        private void RequestStreamCallback(IAsyncResult ar)
        {
            var request = ar.AsyncState as HttpWebRequest;
            request.ContentType = "text/xml";
            var requestStream = request.EndGetRequestStream(ar);
            var streamWriter = new StreamWriter(requestStream);
            var xml = @"
<Product>
  <Discontinued>false</Discontinued> 
  <DiscontinuedDate>0001-01-01T00:00:00</DiscontinuedDate> 
  <ProductId>7</ProductId> 
  <ProductName>Foo</ProductName> 
  <QuantityPerUnit/> 
  <ReorderLevel>0</ReorderLevel> 
  <UnitPrice>0</UnitPrice> 
  <UnitsInStock>0</UnitsInStock> 
  <UnitsOnOrder>0</UnitsOnOrder> 
</Product>";

            streamWriter.Write(xml);
            streamWriter.Close();
            request.BeginGetResponse(new AsyncCallback(ResponseCallback), request);
        }

        private void ResponseCallback(IAsyncResult ar)
        {
            var request = ar.AsyncState as HttpWebRequest;
            var response = request.EndGetResponse(ar);
            var s = response.GetResponseStream();
        }

        void wc_UploadStringCompleted(object sender, UploadStringCompletedEventArgs e)
        {
            var x = e;
        }

        void wc_OpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            using (var streamReader1 = new StreamReader(e.Result))
            {
                var resultString = streamReader1.ReadToEnd();
            }

        }

        //Not used. For show only.
        private void Example_JsonQuery()
        {
            var jsonString = "{\"employees\":[{\"Id\":107,\"FullName\":\"John Papa\"},{\"Id\":108,\"FullName\":\"Kadi Papa\"}]}";
            var employees = JsonObject.Parse(jsonString) as JsonObject;
            var qry = from JsonValue emp in employees["employees"]
                      select new
                                 {
                                     Id = (int)emp["Id"],
                                     Name = (string)emp["FullName"]
                                 };
            foreach (var person in qry)
                Console.WriteLine(person.Name);
        }

        private void Example_DeserializeFromJson()
        {
            // Read it
            var jsonSerializedProduct = "{\"Id\":201,\"ProductName\":\"Some Thing\"}";
            var serializer = new DataContractJsonSerializer(typeof(Product));
            var stream = new MemoryStream(Encoding.UTF8.GetBytes(jsonSerializedProduct));
            var product = serializer.ReadObject(stream) as Product;
            stream.Close();
            var name = product.ProductName;
        }

        private void Example_SerializeToJson()
        {
            // Write it
            var product = new Product { ProductId = 301, ProductName = "Another Thing" };
            var serializer = new DataContractJsonSerializer(typeof(Product));
            var stream = new MemoryStream();
            serializer.WriteObject(stream, product);
            var jsonSerializedProduct = Encoding.UTF8.GetString(stream.ToArray(), 0, (int)stream.Length);

        }
    }
}
